# Stochastic Firm Growth Model with Monte Carlo Parameters
# Code by Blair Fix

#######################################################################
# This code implements a stochastic firm growth model.
# The model assumes scale free growth (Gibrat's Law), with a reflective lower bound and size 1.
# Growth rates are drawn from a laplace distribution.
# Growth is discrete. After every time iteration, firm size is rounded to the nearest integer.
# Growth rate location and scale parameters vary randomly over the chosen interval.
# The model outputs the mean firm size over the last 100 time iterations (and discards model runs that are unstable).


# Packages
################################################
library(doSNOW)
library(foreach)
library(snow)
library(poweRlaw)
library(tcltk)
library(VGAM)
library(ggplot2)

# Clusters
##################################################
cl <- makeCluster(4, type="SOCK")
registerDoSNOW(cl)
clusterSetupRNG (cl, type = "RNGstream")

# Variables
#######################################
# t = the number of time iterations
t = 1000
# n = the number of firms
n = 10^4
# l = the number of Monte Carlo runs over different parameters
l = 100

# Progress Bar
##########################################
startTime <- proc.time()
pb <- txtProgressBar(max=l, style=3)
progress <- function(n) setTxtProgressBar(pb, n)
opts <- list(progress=progress)


# loop over firm size distributions
###############################################################################################
fit =  foreach (h = 1:l, .options.snow=opts,  .combine=rbind, .packages=c('poweRlaw', 'VGAM') ) %dopar%  {


# Initial conditions
##############################################
# f =  a matrix of firm sizes
f = array(0, dim = c(t, n))
# the initial firm size distribution at t = 1 is set to an arbitrary power law distribution
f[1,] = rpldis(n, 1, 3 )

# randomly choose a location parameter between given interval
location = runif(1, 0.9, 1.01)
# randomly choose a scale parameter between given interval
scale = runif(1, 0.05, 0.45)

# Loop over time 
#######################################
for(i in 2:t){
  # gr =  random firm growth rates from laplace distribution (fraction form)
  gr = rlaplace(n, location= location, scale=scale) 
  # negative growth rates are rounded to zero  
  gr[gr <0] = 0                                    
  # new =  size of each firm after growth
  new = round(f[i-1, ]*gr)
  # firms that `died' are reflected to size 1 
  new[new == 0] = 1           
  f[i,] = new
}

# Analysis
#########################################
# size = mean firm size in each time period
size = rowMeans(f)
# t.vec = a time vector at the end of the model run (used to check for stability)
t.vec = (t-100):t
# run a linear regression on mean size vs time
stable = lm(size[t.vec] ~ t.vec)
# stable = the slope of the regression (small indicates model equilibirum)
stable = coef(stable)[2]
# size = mean firm size over the period t.vec
size = mean(size[t.vec])


rm(f, rank, gr, new)
gc()

data.frame(size, stable, location, scale)

}
stopCluster(cl)
close(pb)
endTime <- proc.time()
time = endTime - startTime

####################################################################
# keep only model runs where the mean firm size is stable over time
fit = data.frame(fit)
good = subset(fit, fit$stable <= 0.01)



plot = ggplot() + 
        geom_point(data = good, aes(x = location, y = scale, col= size)) +
        scale_color_gradientn(colours = rainbow(8), trans = "log", limits = c(1,20), breaks = c(1,2,5,10,20)) +
        coord_cartesian(xlim = c(0.9, 0.99), ylim = c(0.05, 0.4))

plot




